/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include "mixer_hctl_proxy.hpp"
#include <QObject>
#include <vector>

namespace MWdg
{

/// @brief Mixer_HCTL_Proxies_Group
///
class Mixer_HCTL_Proxies_Group : public QObject
{
  Q_OBJECT

  public:
  // -- Construction

  Mixer_HCTL_Proxies_Group ( QObject * parent_n );

  ~Mixer_HCTL_Proxies_Group ();

  // -- Proxies

  std::size_t
  num_proxies () const
  {
    return _proxies.size ();
  }

  Mixer_HCTL_Proxy *
  proxy ( std::size_t idx_n ) const
  {
    return _proxies[ idx_n ];
  }

  void
  append_proxy ( Mixer_HCTL_Proxy * proxy_n )
  {
    if ( proxy_n != nullptr ) {
      _proxies.push_back ( proxy_n );
      proxy_n->setParent ( this );
    }
  }

  // Public slots
  Q_SLOT
  void
  update_values ();

  Q_SLOT
  void
  set_joined ( bool flag_n );

  private:
  // -- Attributes
  std::vector< Mixer_HCTL_Proxy * > _proxies;
};

} // namespace MWdg
