/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include "main_window_setup.hpp"
#include "mwdg/mixer_style_db.hpp"
#include "qastools_config.hpp"
#include <QAction>
#include <QActionGroup>
#include <QMainWindow>
#include <QSplitter>
#include <memory>

// Forward declaration
namespace MWdg
{
class Stream_View_Selection;
}
namespace Views
{
class Mixer_Simple;
class Device_Selection_View;
} // namespace Views
namespace dpe
{
class Image_Allocator;
}

/// @brief Main_Window
///
class Main_Window : public QMainWindow
{
  Q_OBJECT

  public:
  // -- Construction

  Main_Window ( dpe::Image_Allocator * image_alloc_n,
                QWidget * parent_n = nullptr,
                Qt::WindowFlags flags_n = Qt::WindowFlags () );

  ~Main_Window ();

  QSize
  sizeHint () const override;

  void
  set_window_setup ( Main_Window_Setup * setup_n );

  void
  select_ctl ( const QString & ctl_n );

  // -- Signals

  Q_SIGNAL
  void
  sig_show_settings ();

  Q_SIGNAL
  void
  sig_show_info ();

  Q_SIGNAL
  void
  sig_control_changed ();

  Q_SIGNAL
  void
  sig_quit ();

  // -- Public slots

  // Snd control selection

  Q_SLOT
  void
  select_ctl_from_side_iface ();

  Q_SLOT
  void
  reload_mixer_device ();

  Q_SLOT
  void
  reload_mixer_inputs ();

  Q_SLOT
  void
  reload_mixer_view ();

  Q_SLOT
  void
  refresh_views ();

  /// @brief Sets/unsets fullscreen mode
  ///
  Q_SLOT
  void
  set_fullscreen ( bool flag_n );

  // Device selection

  Q_SLOT
  void
  show_device_selection ( bool flag_n );

  Q_SLOT
  void
  toggle_device_selection ();

  /// @brief Save state to the setup tree
  ///
  Q_SLOT
  void
  save_state ();

  protected:
  // -- Protected methods

  void
  update_fullscreen_action ();

  // Event handlers

  void
  changeEvent ( QEvent * event_n ) override;

  void
  keyPressEvent ( QKeyEvent * event_n ) override;

  void
  init_menus ();

  void
  init_widgets ();

  private:
  // -- Attributes
  Main_Window_Setup * _win_setup = nullptr;
  MWdg::Mixer_Style_Db _mixer_style_db;
  dpe::Image_Allocator * _image_alloc = nullptr;

  // Base widgets
  std::unique_ptr< QSplitter > _splitter;
  std::unique_ptr< Views::Mixer_Simple > _mixer_simple;
  std::unique_ptr< Views::Device_Selection_View > _dev_select;
  std::unique_ptr< MWdg::Stream_View_Selection > _stream_select;

  // Menubar
  QMenu * _menu_mixer;
  QAction * _act_show_dev_select;
  QAction * _act_fullscreen;

  // Strings and icons
  QString _str_fscreen_enable;
  QString _str_fscreen_disable;
  QIcon _icon_fscreen_enable;
  QIcon _icon_fscreen_disable;
};
