/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include <QMenu>
#include <QPointer>
#include <QTimer>
#include <QWidget>
#include <memory>
#include <vector>

// Forward declaration
namespace MWdg
{
class Inputs_Setup;
class Mixer_Simple_Setup;
class Mixer_Switches_Proxies_Group;
class Mixer_GUI_State_Proxy;
} // namespace MWdg
namespace Wdg
{
class Switches_Pad;
class Pad_Proxy;
class Pad_Proxies_Column;
class Pad_Proxies_Group;
class Switches_Area;
} // namespace Wdg

namespace MWdg
{

/// @brief Mixer_Switches
///
class Mixer_Switches : public QWidget
{
  Q_OBJECT

  public:
  // -- Construction

  Mixer_Switches ( QWidget * parent_n = nullptr );

  ~Mixer_Switches ();

  // Mixer setup

  const Mixer_Simple_Setup *
  mixer_setup () const
  {
    return _mixer_setup;
  }

  void
  set_mixer_setup ( const Mixer_Simple_Setup * setup_n );

  // -- Inputs setup

  const MWdg::Inputs_Setup *
  inputs_setup () const
  {
    return _inputs_setup;
  }

  void
  set_inputs_setup ( const MWdg::Inputs_Setup * setup_n );

  std::size_t
  num_visible () const
  {
    return _proxies_groups_vis.size ();
  }

  // -- Event handling

  bool
  event ( QEvent * event_n ) override;

  // Protected slots
  protected:
  Q_SLOT
  void
  action_toggle_joined ();

  Q_SLOT
  void
  update_focus_proxies ();

  Q_SLOT
  void
  context_menu_cleanup_behind ();

  protected:
  // -- Protected methods

  // Proxy group creation / deletion

  void
  clear_proxies_groups ();

  void
  create_proxies_groups ();

  void
  setup_proxies_group_joined ( MWdg::Mixer_Switches_Proxies_Group * mspg_n );

  void
  setup_proxies_group_separate ( MWdg::Mixer_Switches_Proxies_Group * mspg_n );

  void
  create_proxy ( MWdg::Mixer_Switches_Proxies_Group * mspg_n,
                 int channel_idx_n );

  // Proxy group manipulation

  bool
  should_be_visible ( const MWdg::Mixer_Switches_Proxies_Group * mspg_n ) const;

  void
  toggle_joined_separated ( MWdg::Mixer_Switches_Proxies_Group * mspg_n );

  void
  join_proxies_group ( MWdg::Mixer_Switches_Proxies_Group * mspg_n );

  void
  separate_proxies_group ( MWdg::Mixer_Switches_Proxies_Group * mspg_n );

  // Proxy group showing hiding

  void
  show_visible_proxies_sets ( bool flag_n );

  // Proxy group updating

  void
  separate_where_requested ();

  void
  rebuild_visible_proxies_list ();

  // Focus proxy

  void
  acquire_gui_state ( MWdg::Mixer_GUI_State_Proxy & state_n );

  void
  restore_gui_state ( const MWdg::Mixer_GUI_State_Proxy & state_n );

  MWdg::Mixer_Switches_Proxies_Group *
  find_visible_proxy ( const MWdg::Mixer_GUI_State_Proxy & prox_id_n );

  // Context menu

  bool
  context_menu_start ( const QPoint & pos_n );

  unsigned int
  context_menu_update ();

  // Event callbacks

  bool
  eventFilter ( QObject * watched, QEvent * event ) override;

  /// @brief Switches pad getter
  ///
  const Wdg::Switches_Pad *
  switches_pad () const
  {
    return _switches_pad.get ();
  }

  /// @brief Switches pad getter
  ///
  Wdg::Switches_Pad *
  switches_pad ()
  {
    return _switches_pad.get ();
  }

  private:
  // -- Attributes
  const MWdg::Mixer_Simple_Setup * _mixer_setup = nullptr;
  const MWdg::Inputs_Setup * _inputs_setup = nullptr;

  std::vector< MWdg::Mixer_Switches_Proxies_Group * > _proxies_groups;
  std::vector< MWdg::Mixer_Switches_Proxies_Group * > _proxies_groups_vis;
  std::vector< Wdg::Pad_Proxies_Group * > _proxies_groups_pass;

  std::unique_ptr< Wdg::Switches_Area > _switches_area;
  std::unique_ptr< Wdg::Switches_Pad > _switches_pad;

  // Flags
  bool _separation_requested = false;

  // Action focus proxy
  QPointer< MWdg::Mixer_Switches_Proxies_Group > _act_proxies_group;
  unsigned int _act_proxies_column;

  // Context menu
  QMenu _cmenu;
  QAction _act_toggle_joined;
};

} // namespace MWdg
