/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include "mixer_separation_info.hpp"
#include "mixer_switches_proxy_enum.hpp"
#include "mixer_switches_proxy_switch.hpp"
#include "wdg/pad_proxies_group.hpp"
#include <QObject>
#include <QString>
#include <QTimer>

namespace MWdg
{

/// @brief Mixer_Switches_Proxies_Group
///
class Mixer_Switches_Proxies_Group : public Wdg::Pad_Proxies_Group
{
  Q_OBJECT

  public:
  // -- Construction

  Mixer_Switches_Proxies_Group ( QObject * parent_n = nullptr );

  ~Mixer_Switches_Proxies_Group ();

  // -- Mixer_Simple_Elem

  QSnd::Mixer::Elem *
  mixer_simple_elem () const
  {
    return _mixer_simple_elem;
  }

  virtual void
  set_mixer_simple_elem ( QSnd::Mixer::Elem * selem_n );

  // -- Snd direction

  unsigned char
  snd_dir () const
  {
    return _snd_dir;
  }

  void
  set_snd_dir ( unsigned char dir_n );

  // -- Is joined flag

  bool
  is_joined () const
  {
    return _is_joined;
  }

  void
  set_is_joined ( bool flag_n );

  // -- Is visible flag

  bool
  is_visible () const
  {
    return _is_visible;
  }

  void
  set_is_visible ( bool flag_n );

  // -- Notify value change

  bool
  notify_value_change () const
  {
    return _notify_value_change;
  }

  void
  set_notify_value_change ( bool flag_n );

  // -- Separation info

  bool
  can_be_separated () const;

  bool
  should_be_separated () const;

  bool
  needs_separation () const;

  bool
  separation_request () const
  {
    return _separation_request;
  }

  void
  set_separation_request ( bool flag_n );

  // -- Public slots

  Q_SLOT
  void
  update_mixer_values ();

  // -- Protected slots
  protected:
  Q_SLOT
  void
  timer_separation_check ();

  private:
  // -- Attributes
  QSnd::Mixer::Elem * _mixer_simple_elem;

  unsigned char _snd_dir;
  bool _is_joined;
  bool _is_visible;
  bool _notify_value_change;
  bool _separation_request;

  QTimer _separation_timer;
};

} // namespace MWdg
