/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include "mwdg/mixer_style_db.hpp"
#include "wdg/balloon_widget.hpp"
#include "wdg/label_width.hpp"
#include <QLabel>
#include <QPixmap>
#include <QStackedLayout>
#include <QString>

// Forward declaration
namespace dpe
{
class Image_Allocator;
}

/// @brief Tray_Mixer_Balloon
///
class Tray_Mixer_Balloon : public Wdg::Balloon_Widget
{
  Q_OBJECT

  public:
  // -- Construction

  Tray_Mixer_Balloon ( dpe::Image_Allocator * image_alloc_n,
                       QWidget * parent_n = nullptr );

  ~Tray_Mixer_Balloon ();

  QSize
  pixmap_size () const;

  void
  set_image_alloc ( dpe::Image_Allocator * image_alloc_n );

  // -- Volume value

  void
  set_muted ( bool flag_n );

  void
  set_permille ( unsigned int value_n );

  void
  set_icon ( const QIcon & icon_n );

  void
  set_pixmap ( const QPixmap & pixmap_n );

  bool
  eventFilter ( QObject * obj_n, QEvent * event_n ) override;

  // -- Signals

  Q_SIGNAL
  void
  sig_activated ();

  Q_SIGNAL
  void
  sig_middle_click ();

  Q_SIGNAL
  void
  sig_wheel_delta ( int delta_n );

  protected:
  // -- Protected methods

  void
  mousePressEvent ( QMouseEvent * event_n ) override;

  void
  changeEvent ( QEvent * event_n ) override;

  private:
  // -- Attributes
  QLabel * _lbl_icon;
  QStackedLayout * _lay_volume_stack;
  Wdg::Label_Width * _lbl_volume_value;
  QLabel * _lbl_volume_muted;

  dpe::Image_Allocator * _image_alloc;
  MWdg::Mixer_Style_Db _mixer_style_db;

  // Strings
  QString _value_mask;
};
