/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include "mwdg/mixer_simple_setup.hpp"
#include "views/view_base.hpp"
#include <QPointer>
#include <QSplitter>

// Forward declaration
namespace QSnd
{
class Mixer_Simple;
}
namespace MWdg
{
class Mixer_Sliders;
class Mixer_Sliders_Status_Widget;
class Mixer_Switches;
} // namespace MWdg
namespace Views
{
class Mixer_Simple_Setup;
}
namespace Wdg::DS
{
class Widget_Style_Db;
}
namespace dpe
{
class Image_Allocator;
}

namespace Views
{

/// @brief Mixer_Simple
///
class Mixer_Simple : public Views::View_Base
{
  Q_OBJECT

  public:
  // -- Construction

  Mixer_Simple ( Wdg::DS::Widget_Style_Db * wdg_style_db_n,
                 dpe::Image_Allocator * image_alloc_n,
                 QWidget * parent_n = nullptr );

  ~Mixer_Simple ();

  // -- Mixer device and view setup

  void
  set_mdev_setup ( const MWdg::Mixer_Device_Setup * setup_n );

  void
  set_inputs_setup ( const MWdg::Inputs_Setup * setup_n );

  void
  set_view_setup ( Views::View_Base_Setup * setup_n );

  // -- Streams available

  Q_SIGNAL
  void
  sig_streams_available ( bool value_n );

  // -- Playback

  Q_SLOT
  void
  show_playback ( bool flag_n );

  Q_SLOT
  void
  toggle_show_playback ();

  Q_SIGNAL
  void
  sig_show_playback_changed ( bool value_n );

  // -- Capture

  Q_SLOT
  void
  show_capture ( bool flag_n );

  Q_SLOT
  void
  toggle_show_capture ();

  Q_SIGNAL
  void
  sig_show_capture_changed ( bool value_n );

  // -- Slider value widget

  Q_SLOT
  void
  show_slider_value_widget ();

  // -- Protected slots
  protected:
  Q_SLOT
  void
  footer_label_selected ( unsigned int group_idx_n, unsigned int column_idx_n );

  protected:
  // -- Protected methods

  void
  keyPressEvent ( QKeyEvent * event_n ) override;

  void
  showEvent ( QShowEvent * event_n ) override;

  void
  hideEvent ( QHideEvent * event_n ) override;

  private:
  // -- Private methods

  void
  clear_view ();

  void
  setup_view ();

  private:
  // -- Attributes
  Views::Mixer_Simple_Setup * _view_setup = nullptr;
  MWdg::Mixer_Simple_Setup _smixer_setup;

  // widgets
  QWidget _wdg_areas;
  QSplitter _mixer_split;
  MWdg::Mixer_Sliders * _mixer_sliders;
  MWdg::Mixer_Switches * _mixer_switches;

  QPointer< MWdg::Mixer_Sliders_Status_Widget > _status_wdg;
  unsigned int _status_group_idx = ~0;
  unsigned int _status_column_idx = ~0;

  /// @brief Mixer object
  QSnd::Mixer::Mixer * _qsnd_mixer = nullptr;
};

} // namespace Views
