/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include "qsnd/ctl_address.hpp"
#include <QVBoxLayout>
#include <QWidget>
#include <memory>

// Forward declaration
class QModelIndex;
namespace MWdg
{
class Mixers_View;
class User_Device_Input;
} // namespace MWdg
namespace QSnd
{
class Mixers_Model;
}
namespace Views
{
class Device_Selection_View_Setup;
}

namespace Views
{

/// @brief Device_Selection_View
///
/// Mixer device selection view
class Device_Selection_View : public QWidget
{
  Q_OBJECT

  public:
  // Construction

  Device_Selection_View ( QWidget * parent_n = nullptr );

  ~Device_Selection_View ();

  void
  set_view_setup ( Views::Device_Selection_View_Setup * setup_n );

  /// @brief Currently selected control address
  const QSnd::CTL_Address &
  selected_ctl () const
  {
    return _selected_ctl;
  }

  void
  silent_select_ctl ( const QSnd::CTL_Address & ctl_addr_n );

  // Signals

  Q_SIGNAL
  void
  sig_close ();

  Q_SIGNAL
  void
  sig_control_selected ();

  // Public slots

  Q_SLOT
  void
  reload_database ();

  // Layout access

  QVBoxLayout *
  lay_bottom ()
  {
    return _lay_bottom;
  }

  protected:
  // -- Protected methods

  void
  contextMenuEvent ( QContextMenuEvent * event_n ) override;

  private:
  // -- Private methods

  Q_SLOT
  void
  control_selection_changed ();

  QSnd::CTL_Address
  acquire_ctl_address () const;

  void
  update_selected_ctl ();

  void
  user_device_input_update ();

  Q_SLOT
  void
  user_device_input_changed ();

  private:
  // -- Attributes
  Views::Device_Selection_View_Setup * _view_setup = nullptr;

  std::unique_ptr< QSnd::Mixers_Model > _mixers_model;
  std::unique_ptr< MWdg::Mixers_View > _mixers_view;
  std::unique_ptr< MWdg::User_Device_Input > _user_device_input;

  // Selection state
  QSnd::CTL_Address _selected_ctl;
  bool _silent_ctl_change = false;

  // Layout
  QVBoxLayout * _lay_user_device = nullptr;
  QVBoxLayout * _lay_bottom = nullptr;

  // Context menu
  QAction * _act_close = nullptr;
};

} // namespace Views
