/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include <QFrame>
#include <QTimer>

namespace Wdg
{

/// @brief Balloon_Widget
///
class Balloon_Widget : public QWidget
{
  Q_OBJECT

  public:
  // -- Construction

  Balloon_Widget ( QWidget * parent_n = nullptr );

  ~Balloon_Widget ();

  // -- Duration

  unsigned int
  duration_ms () const;

  void
  set_duration_ms ( unsigned int ms_n );

  /// @brief Stay visible if widget is under mouse
  ///
  /// @return The flag - true by default
  bool
  stay_on_hover () const
  {
    return _stay_on_hover;
  }

  void
  set_stay_on_hover ( bool flag_n );

  // -- Tray icon geometry

  const QRect &
  tray_icon_geometry () const
  {
    return _tray_icon_geom;
  }

  void
  set_tray_icon_geometry ( const QRect & geom_n );

  // -- Widgets

  void
  add_widget ( QWidget * wdg_n );

  // -- Public signals

  Q_SIGNAL
  void
  sig_close ();

  // -- Public slots

  Q_SLOT
  void
  start_show ();

  protected:
  // -- Protected methods

  void
  leaveEvent ( QEvent * event_n ) override;

  void
  resizeEvent ( QResizeEvent * event_n ) override;

  void
  paintEvent ( QPaintEvent * event_n ) override;

  private:
  // -- Private slots

  Q_SLOT
  void
  close_timeout ();

  private:
  // -- Private methods

  void
  update_mask ();

  void
  update_pixmap ();

  void
  update_geometry ();

  private:
  // -- Attributes
  QPixmap _pxmap;
  QRect _tray_icon_geom;
  QTimer _close_timer;

  static const unsigned int _border_width = 1;
  static const unsigned int _corner_rad = 6;

  bool _update_pxmap = true;
  bool _stay_on_hover = true;
  bool _remains_on_hover = false;
};

} // namespace Wdg
