/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#pragma once

#include <QLayoutItem>

namespace Wdg
{

/// @brief Equal_Columns_Layout_Column
///
class Equal_Columns_Layout_Row
{
  public:
  // -- Construction

  Equal_Columns_Layout_Row ( unsigned int row_idx_n );

  ~Equal_Columns_Layout_Row ();

  // -- Row index

  unsigned int
  row_index () const
  {
    return _row_index;
  }

  void
  set_row_index ( unsigned int row_idx_n )
  {
    _row_index = row_idx_n;
  }

  // -- Layout item

  QLayoutItem *
  item () const
  {
    return _item;
  }

  void
  set_item ( QLayoutItem * item_n )
  {
    _item = item_n;
  }

  private:
  // -- Attributes
  unsigned int _row_index = 0;
  QLayoutItem * _item = nullptr;
};

/// @brief Equal_Columns_Layout_Column
///
class Equal_Columns_Layout_Column
{
  public:
  // -- Construction

  Equal_Columns_Layout_Column ( unsigned int column_idx_n );

  virtual ~Equal_Columns_Layout_Column ();

  // Column index

  unsigned int
  column_index () const
  {
    return _column_index;
  }

  void
  set_column_index ( unsigned int idx_n )
  {
    _column_index = idx_n;
  }

  // Active rows

  unsigned int
  active_rows () const
  {
    return _active_rows;
  }

  void
  set_active_rows ( unsigned int num_n )
  {
    _active_rows = num_n;
  }

  // Column position

  unsigned int
  column_pos () const
  {
    return _column_pos;
  }

  void
  set_column_pos ( unsigned int pos_n );

  // Column width

  unsigned int
  column_width () const
  {
    return _column_width;
  }

  void
  set_column_width ( unsigned int pos_n );

  // Rows

  void
  clear_rows ();

  void
  append_row ( Equal_Columns_Layout_Row * item_n );

  void
  remove_empty_rows_at_back ();

  unsigned int
  num_rows () const
  {
    return _rows.size ();
  }

  const Equal_Columns_Layout_Row *
  row ( std::size_t idx_n ) const
  {
    return _rows[ idx_n ];
  }

  Equal_Columns_Layout_Row *
  row ( std::size_t idx_n )
  {
    return _rows[ idx_n ];
  }

  private:
  // -- Attributes
  unsigned int _column_index;
  unsigned int _active_rows;
  unsigned int _column_pos;
  unsigned int _column_width;
  std::vector< Equal_Columns_Layout_Row * > _rows;
};

/// @brief Equal_Columns_Layout_Group
///
class Equal_Columns_Layout_Group
{
  public:
  // -- Construction

  Equal_Columns_Layout_Group ( unsigned int group_idx_n );

  virtual ~Equal_Columns_Layout_Group ();

  // -- Group index

  unsigned int
  group_index () const
  {
    return _group_index;
  }

  void
  set_group_index ( unsigned int idx_n )
  {
    _group_index = idx_n;
  }

  // -- Active columns

  unsigned int
  active_columns () const
  {
    return _active_columns;
  }

  void
  set_active_columns ( unsigned int num_n )
  {
    _active_columns = num_n;
  }

  // -- Group pos

  unsigned int
  group_pos () const
  {
    return _group_pos;
  }

  void
  set_group_pos ( unsigned int pos_n );

  // -- Group width

  unsigned int
  group_width () const
  {
    return _group_width;
  }

  void
  set_group_width ( unsigned int pos_n );

  // -- Columns

  void
  clear_columns ();

  void
  append_column ( Equal_Columns_Layout_Column * column_n );

  void
  remove_empty_columns_at_back ();

  std::size_t
  num_columns () const
  {
    return _columns.size ();
  }

  const Equal_Columns_Layout_Column *
  column ( std::size_t idx_n ) const
  {
    return _columns[ idx_n ];
  }

  Equal_Columns_Layout_Column *
  column ( std::size_t idx_n )
  {
    return _columns[ idx_n ];
  }

  // -- Row statistics

  std::vector< unsigned int > &
  row_stats ()
  {
    return _row_stats;
  }

  std::size_t
  num_rows () const
  {
    return _row_stats.size ();
  }

  unsigned int
  row_stat ( std::size_t idx_n ) const
  {
    return _row_stats[ idx_n ];
  }

  private:
  // -- Attributes
  unsigned int _group_index;
  unsigned int _active_columns;
  unsigned int _group_pos;
  unsigned int _group_width;
  std::vector< Equal_Columns_Layout_Column * > _columns;
  std::vector< unsigned int > _row_stats;
};

} // namespace Wdg
